@@ -17,19 +17,26 @@ async def async_setup_entry(hass, config_entry, async_add_entities):
     @callback
     def async_add_switch(event_id):
         """Add switch from Axis device."""
-        event = device.api.event.events[event_id]
+        event = device.api.event[event_id]
 
         if event.CLASS == CLASS_OUTPUT:
-            async_add_entities([AxisSwitch(event, device)], True)
+            async_add_entities([AxisSwitch(event, device)])
 
-    device.listeners.append(
-        async_dispatcher_connect(hass, device.event_new_sensor, async_add_switch)
+    config_entry.async_on_unload(
+        async_dispatcher_connect(hass, device.signal_new_event, async_add_switch)
     )
 
 
 class AxisSwitch(AxisEventBase, SwitchEntity):
     """Representation of a Axis switch."""
 
+    def __init__(self, event, device):
+        """Initialize the Axis switch."""
+        super().__init__(event, device)
+
+        if event.id and device.api.vapix.ports[event.id].name:
+            self._attr_name = f"{device.name} {device.api.vapix.ports[event.id].name}"
+
     @property
     def is_on(self):
         """Return true if event is active."""
@@ -37,24 +44,8 @@ class AxisSwitch(AxisEventBase, SwitchEntity):
 
     async def async_turn_on(self, **kwargs):
         """Turn on switch."""
-        action = "/"
-        await self.hass.async_add_executor_job(
-            self.device.api.vapix.ports[self.event.id].action, action
-        )
+        await self.device.api.vapix.ports[self.event.id].close()
 
     async def async_turn_off(self, **kwargs):
         """Turn off switch."""
-        action = "\\"
-        await self.hass.async_add_executor_job(
-            self.device.api.vapix.ports[self.event.id].action, action
-        )
-
-    @property
-    def name(self):
-        """Return the name of the event."""
-        if self.event.id and self.device.api.vapix.ports[self.event.id].name:
-            return (
-                f"{self.device.name} {self.device.api.vapix.ports[self.event.id].name}"
-            )
-
-        return super().name
+        await self.device.api.vapix.ports[self.event.id].open()
