@@ -1,4 +1,6 @@
 """Support for Apple HomeKit."""
+from __future__ import annotations
+
 import asyncio
 import ipaddress
 import logging
@@ -8,7 +10,7 @@ from aiohttp import web
 from pyhap.const import STANDALONE_AID
 import voluptuous as vol
 
-from homeassistant.components import zeroconf
+from homeassistant.components import device_automation, network, zeroconf
 from homeassistant.components.binary_sensor import (
     DEVICE_CLASS_BATTERY_CHARGING,
     DEVICE_CLASS_MOTION,
@@ -18,12 +20,18 @@ from homeassistant.components.binary_sensor import (
 from homeassistant.components.camera import DOMAIN as CAMERA_DOMAIN
 from homeassistant.components.http import HomeAssistantView
 from homeassistant.components.humidifier import DOMAIN as HUMIDIFIER_DOMAIN
+from homeassistant.components.network.const import MDNS_TARGET_IP
 from homeassistant.components.sensor import DOMAIN as SENSOR_DOMAIN
 from homeassistant.config_entries import SOURCE_IMPORT, ConfigEntry
 from homeassistant.const import (
     ATTR_BATTERY_CHARGING,
     ATTR_BATTERY_LEVEL,
+    ATTR_DEVICE_ID,
     ATTR_ENTITY_ID,
+    ATTR_MANUFACTURER,
+    ATTR_MODEL,
+    ATTR_SW_VERSION,
+    CONF_DEVICES,
     CONF_IP_ADDRESS,
     CONF_NAME,
     CONF_PORT,
@@ -34,15 +42,15 @@ from homeassistant.const import (
     SERVICE_RELOAD,
 )
 from homeassistant.core import CoreState, HomeAssistant, callback
-from homeassistant.exceptions import ConfigEntryNotReady, Unauthorized
+from homeassistant.exceptions import HomeAssistantError, Unauthorized
 from homeassistant.helpers import device_registry, entity_registry
 import homeassistant.helpers.config_validation as cv
 from homeassistant.helpers.entityfilter import BASE_FILTER_SCHEMA, FILTER_SCHEMA
 from homeassistant.helpers.reload import async_integration_yaml_config
+from homeassistant.helpers.service import async_extract_referenced_entity_ids
+from homeassistant.helpers.typing import ConfigType
 from homeassistant.loader import IntegrationNotFound, async_get_integration
-from homeassistant.util import get_local_ip
 
-# pylint: disable=unused-import
 from . import (  # noqa: F401
     type_cameras,
     type_covers,
@@ -51,6 +59,7 @@ from . import (  # noqa: F401
     type_lights,
     type_locks,
     type_media_players,
+    type_remotes,
     type_security_systems,
     type_sensors,
     type_switches,
@@ -59,15 +68,10 @@ from . import (  # noqa: F401
 from .accessories import HomeBridge, HomeDriver, get_accessory
 from .aidmanager import AccessoryAidStorage
 from .const import (
-    AID_STORAGE,
-    ATTR_INTERGRATION,
-    ATTR_MANUFACTURER,
-    ATTR_MODEL,
-    ATTR_SOFTWARE_VERSION,
+    ATTR_INTEGRATION,
     BRIDGE_NAME,
     BRIDGE_SERIAL_NUMBER,
     CONF_ADVERTISE_IP,
-    CONF_AUTO_START,
     CONF_ENTITY_CONFIG,
     CONF_ENTRY_INDEX,
     CONF_EXCLUDE_ACCESSORY_MODE,
@@ -78,14 +82,10 @@ from .const import (
     CONF_LINKED_DOORBELL_SENSOR,
     CONF_LINKED_HUMIDITY_SENSOR,
     CONF_LINKED_MOTION_SENSOR,
-    CONF_SAFE_MODE,
-    CONF_ZEROCONF_DEFAULT_INTERFACE,
     CONFIG_OPTIONS,
-    DEFAULT_AUTO_START,
     DEFAULT_EXCLUDE_ACCESSORY_MODE,
     DEFAULT_HOMEKIT_MODE,
     DEFAULT_PORT,
-    DEFAULT_SAFE_MODE,
     DOMAIN,
     HOMEKIT,
     HOMEKIT_MODE_ACCESSORY,
@@ -93,18 +93,20 @@ from .const import (
     HOMEKIT_PAIRING_QR,
     HOMEKIT_PAIRING_QR_SECRET,
     MANUFACTURER,
+    PERSIST_LOCK,
     SERVICE_HOMEKIT_RESET_ACCESSORY,
     SERVICE_HOMEKIT_START,
+    SERVICE_HOMEKIT_UNPAIR,
     SHUTDOWN_TIMEOUT,
-    UNDO_UPDATE_LISTENER,
 )
+from .type_triggers import DeviceTriggerAccessory
 from .util import (
     accessory_friendly_name,
-    dismiss_setup_message,
+    async_dismiss_setup_message,
+    async_port_is_available,
+    async_show_setup_message,
     get_persist_fullpath_for_entry_id,
-    port_is_available,
     remove_state_files_for_entry_id,
-    show_setup_message,
     state_needs_accessory_mode,
     validate_entity_config,
 )
@@ -119,6 +121,12 @@ STATUS_RUNNING = 1
 STATUS_STOPPED = 2
 STATUS_WAIT = 3
 
+PORT_CLEANUP_CHECK_INTERVAL_SECS = 1
+
+_HOMEKIT_CONFIG_UPDATE_TIME = (
+    5  # number of seconds to wait for homekit to see the c# change
+)
+
 
 def _has_all_unique_names_and_ports(bridges):
     """Validate that each homekit bridge configured has a unique name."""
@@ -130,8 +138,6 @@ def _has_all_unique_names_and_ports(bridges):
 
 
 BRIDGE_SCHEMA = vol.All(
-    cv.deprecated(CONF_ZEROCONF_DEFAULT_INTERFACE),
-    cv.deprecated(CONF_SAFE_MODE),
     vol.Schema(
         {
             vol.Optional(CONF_HOMEKIT_MODE, default=DEFAULT_HOMEKIT_MODE): vol.In(
@@ -143,11 +149,9 @@ BRIDGE_SCHEMA = vol.All(
             vol.Optional(CONF_PORT, default=DEFAULT_PORT): cv.port,
             vol.Optional(CONF_IP_ADDRESS): vol.All(ipaddress.ip_address, cv.string),
             vol.Optional(CONF_ADVERTISE_IP): vol.All(ipaddress.ip_address, cv.string),
-            vol.Optional(CONF_AUTO_START, default=DEFAULT_AUTO_START): cv.boolean,
-            vol.Optional(CONF_SAFE_MODE, default=DEFAULT_SAFE_MODE): cv.boolean,
             vol.Optional(CONF_FILTER, default={}): BASE_FILTER_SCHEMA,
             vol.Optional(CONF_ENTITY_CONFIG, default={}): validate_entity_config,
-            vol.Optional(CONF_ZEROCONF_DEFAULT_INTERFACE): cv.boolean,
+            vol.Optional(CONF_DEVICES): cv.ensure_list,
         },
         extra=vol.ALLOW_EXTRA,
     ),
@@ -164,6 +168,21 @@ RESET_ACCESSORY_SERVICE_SCHEMA = vol.Schema(
 )
 
 
+UNPAIR_SERVICE_SCHEMA = vol.All(
+    vol.Schema(cv.ENTITY_SERVICE_FIELDS),
+    cv.has_at_least_one_key(ATTR_DEVICE_ID),
+)
+
+
+def _async_all_homekit_instances(hass: HomeAssistant) -> list[HomeKit]:
+    """All active HomeKit instances."""
+    return [
+        data[HOMEKIT]
+        for data in hass.data[DOMAIN].values()
+        if isinstance(data, dict) and HOMEKIT in data
+    ]
+
+
 def _async_get_entries_by_name(current_entries):
     """Return a dict of the entries by name."""
 
@@ -172,9 +191,9 @@ def _async_get_entries_by_name(current_entries):
     return {entry.data.get(CONF_NAME, BRIDGE_NAME): entry for entry in current_entries}
 
 
-async def async_setup(hass: HomeAssistant, config: dict):
+async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool:
     """Set up the HomeKit from yaml."""
-    hass.data.setdefault(DOMAIN, {})
+    hass.data.setdefault(DOMAIN, {})[PERSIST_LOCK] = asyncio.Lock()
 
     _async_register_events_and_services(hass)
 
@@ -221,8 +240,9 @@ def _async_update_config_entry_if_from_yaml(hass, entries_by_name, conf):
         data = conf.copy()
         options = {}
         for key in CONFIG_OPTIONS:
-            options[key] = data[key]
-            del data[key]
+            if key in data:
+                options[key] = data[key]
+                del data[key]
 
         hass.config_entries.async_update_entry(entry, data=data, options=options)
         return True
@@ -230,7 +250,7 @@ def _async_update_config_entry_if_from_yaml(hass, entries_by_name, conf):
     return False
 
 
-async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry):
+async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Set up HomeKit from a config entry."""
     _async_import_options_from_data_if_missing(hass, entry)
 
@@ -241,11 +261,10 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry):
     port = conf[CONF_PORT]
     _LOGGER.debug("Begin setup HomeKit for %s", name)
 
-    aid_storage = AccessoryAidStorage(hass, entry.entry_id)
-
-    await aid_storage.async_initialize()
     # ip_address and advertise_ip are yaml only
-    ip_address = conf.get(CONF_IP_ADDRESS)
+    ip_address = conf.get(
+        CONF_IP_ADDRESS, await network.async_get_source_ip(hass, MDNS_TARGET_IP)
+    )
     advertise_ip = conf.get(CONF_ADVERTISE_IP)
     # exclude_accessory_mode is only used for config flow
     # to indicate that the config entry was setup after
@@ -260,8 +279,8 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry):
     )
     homekit_mode = options.get(CONF_HOMEKIT_MODE, DEFAULT_HOMEKIT_MODE)
     entity_config = options.get(CONF_ENTITY_CONFIG, {}).copy()
-    auto_start = options.get(CONF_AUTO_START, DEFAULT_AUTO_START)
     entity_filter = FILTER_SCHEMA(options.get(CONF_FILTER, {}))
+    devices = options.get(CONF_DEVICES, [])
 
     homekit = HomeKit(
         hass,
@@ -275,30 +294,19 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry):
         advertise_ip,
         entry.entry_id,
         entry.title,
+        devices=devices,
     )
-    zeroconf_instance = await zeroconf.async_get_instance(hass)
 
-    # If the previous instance hasn't cleaned up yet
-    # we need to wait a bit
-    try:
-        await hass.async_add_executor_job(homekit.setup, zeroconf_instance)
-    except (OSError, AttributeError) as ex:
-        _LOGGER.warning(
-            "%s could not be setup because the local port %s is in use", name, port
-        )
-        raise ConfigEntryNotReady from ex
+    entry.async_on_unload(entry.add_update_listener(_async_update_listener))
+    entry.async_on_unload(
+        hass.bus.async_listen_once(EVENT_HOMEASSISTANT_STOP, homekit.async_stop)
+    )
 
-    undo_listener = entry.add_update_listener(_async_update_listener)
-
-    hass.data[DOMAIN][entry.entry_id] = {
-        AID_STORAGE: aid_storage,
-        HOMEKIT: homekit,
-        UNDO_UPDATE_LISTENER: undo_listener,
-    }
+    hass.data[DOMAIN][entry.entry_id] = {HOMEKIT: homekit}
 
     if hass.state == CoreState.running:
         await homekit.async_start()
-    elif auto_start:
+    else:
         hass.bus.async_listen_once(EVENT_HOMEASSISTANT_STARTED, homekit.async_start)
 
     return True
@@ -311,23 +319,24 @@ async def _async_update_listener(hass: HomeAssistant, entry: ConfigEntry):
     await hass.config_entries.async_reload(entry.entry_id)
 
 
-async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry):
+async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Unload a config entry."""
-    dismiss_setup_message(hass, entry.entry_id)
-
-    hass.data[DOMAIN][entry.entry_id][UNDO_UPDATE_LISTENER]()
-
+    async_dismiss_setup_message(hass, entry.entry_id)
     homekit = hass.data[DOMAIN][entry.entry_id][HOMEKIT]
 
     if homekit.status == STATUS_RUNNING:
         await homekit.async_stop()
 
+    logged_shutdown_wait = False
     for _ in range(0, SHUTDOWN_TIMEOUT):
-        if not await hass.async_add_executor_job(
-            port_is_available, entry.data[CONF_PORT]
-        ):
+        if async_port_is_available(entry.data[CONF_PORT]):
+            break
+
+        if not logged_shutdown_wait:
             _LOGGER.info("Waiting for the HomeKit server to shutdown")
-            await asyncio.sleep(1)
+            logged_shutdown_wait = True
+
+        await asyncio.sleep(PORT_CLEANUP_CHECK_INTERVAL_SECS)
 
     hass.data[DOMAIN].pop(entry.entry_id)
 
@@ -361,12 +370,9 @@ def _async_register_events_and_services(hass: HomeAssistant):
     """Register events and services for HomeKit."""
     hass.http.register_view(HomeKitPairingQRView)
 
-    def handle_homekit_reset_accessory(service):
-        """Handle start HomeKit service call."""
-        for entry_id in hass.data[DOMAIN]:
-            if HOMEKIT not in hass.data[DOMAIN][entry_id]:
-                continue
-            homekit = hass.data[DOMAIN][entry_id][HOMEKIT]
+    async def async_handle_homekit_reset_accessory(service):
+        """Handle reset accessory HomeKit service call."""
+        for homekit in _async_all_homekit_instances(hass):
             if homekit.status != STATUS_RUNNING:
                 _LOGGER.warning(
                     "HomeKit is not running. Either it is waiting to be "
@@ -375,22 +381,51 @@ def _async_register_events_and_services(hass: HomeAssistant):
                 continue
 
             entity_ids = service.data.get("entity_id")
-            homekit.reset_accessories(entity_ids)
+            await homekit.async_reset_accessories(entity_ids)
 
     hass.services.async_register(
         DOMAIN,
         SERVICE_HOMEKIT_RESET_ACCESSORY,
-        handle_homekit_reset_accessory,
+        async_handle_homekit_reset_accessory,
         schema=RESET_ACCESSORY_SERVICE_SCHEMA,
     )
 
+    async def async_handle_homekit_unpair(service):
+        """Handle unpair HomeKit service call."""
+        referenced = async_extract_referenced_entity_ids(hass, service)
+        dev_reg = device_registry.async_get(hass)
+        for device_id in referenced.referenced_devices:
+            if not (dev_reg_ent := dev_reg.async_get(device_id)):
+                raise HomeAssistantError(f"No device found for device id: {device_id}")
+            macs = [
+                cval
+                for ctype, cval in dev_reg_ent.connections
+                if ctype == device_registry.CONNECTION_NETWORK_MAC
+            ]
+            matching_instances = [
+                homekit
+                for homekit in _async_all_homekit_instances(hass)
+                if homekit.driver
+                and device_registry.format_mac(homekit.driver.state.mac) in macs
+            ]
+            if not matching_instances:
+                raise HomeAssistantError(
+                    f"No homekit accessory found for device id: {device_id}"
+                )
+            for homekit in matching_instances:
+                homekit.async_unpair()
+
+    hass.services.async_register(
+        DOMAIN,
+        SERVICE_HOMEKIT_UNPAIR,
+        async_handle_homekit_unpair,
+        schema=UNPAIR_SERVICE_SCHEMA,
+    )
+
     async def async_handle_homekit_service_start(service):
         """Handle start HomeKit service call."""
         tasks = []
-        for entry_id in hass.data[DOMAIN]:
-            if HOMEKIT not in hass.data[DOMAIN][entry_id]:
-                continue
-            homekit = hass.data[DOMAIN][entry_id][HOMEKIT]
+        for homekit in _async_all_homekit_instances(hass):
             if homekit.status == STATUS_RUNNING:
                 _LOGGER.debug("HomeKit is already running")
                 continue
@@ -450,6 +485,7 @@ class HomeKit:
         advertise_ip=None,
         entry_id=None,
         entry_title=None,
+        devices=None,
     ):
         """Initialize a HomeKit object."""
         self.hass = hass
@@ -463,15 +499,15 @@ class HomeKit:
         self._entry_id = entry_id
         self._entry_title = entry_title
         self._homekit_mode = homekit_mode
+        self._devices = devices or []
+        self.aid_storage = None
         self.status = STATUS_READY
 
         self.bridge = None
         self.driver = None
 
-    def setup(self, zeroconf_instance):
+    def setup(self, async_zeroconf_instance, uuid):
         """Set up bridge and accessory driver."""
-        self.hass.bus.async_listen_once(EVENT_HOMEASSISTANT_STOP, self.async_stop)
-        ip_addr = self._ip_address or get_local_ip()
         persist_file = get_persist_fullpath_for_entry_id(self.hass, self._entry_id)
 
         self.driver = HomeDriver(
@@ -480,67 +516,81 @@ class HomeKit:
             self._name,
             self._entry_title,
             loop=self.hass.loop,
-            address=ip_addr,
+            address=self._ip_address,
             port=self._port,
             persist_file=persist_file,
             advertised_address=self._advertise_ip,
-            zeroconf_instance=zeroconf_instance,
+            async_zeroconf_instance=async_zeroconf_instance,
+            zeroconf_server=f"{uuid}-hap.local.",
         )
 
         # If we do not load the mac address will be wrong
         # as pyhap uses a random one until state is restored
         if os.path.exists(persist_file):
             self.driver.load()
-            self.driver.state.config_version += 1
-            if self.driver.state.config_version > 65535:
-                self.driver.state.config_version = 1
 
-        self.driver.persist()
-
-    def reset_accessories(self, entity_ids):
+    async def async_reset_accessories(self, entity_ids):
         """Reset the accessory to load the latest configuration."""
         if not self.bridge:
-            self.driver.config_changed()
+            await self.async_reset_accessories_in_accessory_mode(entity_ids)
             return
+        await self.async_reset_accessories_in_bridge_mode(entity_ids)
 
-        aid_storage = self.hass.data[DOMAIN][self._entry_id][AID_STORAGE]
-        removed = []
+    async def async_reset_accessories_in_accessory_mode(self, entity_ids):
+        """Reset accessories in accessory mode."""
+        acc = self.driver.accessory
+        if acc.entity_id not in entity_ids:
+            return
+        await acc.stop()
+        if not (state := self.hass.states.get(acc.entity_id)):
+            _LOGGER.warning(
+                "The underlying entity %s disappeared during reset", acc.entity
+            )
+            return
+        if new_acc := self._async_create_single_accessory([state]):
+            self.driver.accessory = new_acc
+            self.hass.async_add_job(new_acc.run)
+            await self.async_config_changed()
+
+    async def async_reset_accessories_in_bridge_mode(self, entity_ids):
+        """Reset accessories in bridge mode."""
+        new = []
         for entity_id in entity_ids:
-            aid = aid_storage.get_or_allocate_aid_for_entity_id(entity_id)
+            aid = self.aid_storage.get_or_allocate_aid_for_entity_id(entity_id)
             if aid not in self.bridge.accessories:
                 continue
-
             _LOGGER.info(
                 "HomeKit Bridge %s will reset accessory with linked entity_id %s",
                 self._name,
                 entity_id,
             )
+            acc = await self.async_remove_bridge_accessory(aid)
+            if state := self.hass.states.get(acc.entity_id):
+                new.append(state)
+            else:
+                _LOGGER.warning(
+                    "The underlying entity %s disappeared during reset", acc.entity
+                )
 
-            acc = self.remove_bridge_accessory(aid)
-            removed.append(acc)
-
-        if not removed:
+        if not new:
             # No matched accessories, probably on another bridge
             return
 
-        self.driver.config_changed()
+        await self.async_config_changed()
+        await asyncio.sleep(_HOMEKIT_CONFIG_UPDATE_TIME)
+        for state in new:
+            acc = self.add_bridge_accessory(state)
+            if acc:
+                self.hass.async_add_job(acc.run)
+        await self.async_config_changed()
 
-        for acc in removed:
-            self.bridge.add_accessory(acc)
-        self.driver.config_changed()
+    async def async_config_changed(self):
+        """Call config changed which writes out the new config to disk."""
+        await self.hass.async_add_executor_job(self.driver.config_changed)
 
     def add_bridge_accessory(self, state):
         """Try adding accessory to bridge if configured beforehand."""
-        if not self._filter(state.entity_id):
-            return
-
-        # The bridge itself counts as an accessory
-        if len(self.bridge.accessories) + 1 >= MAX_DEVICES:
-            _LOGGER.warning(
-                "Cannot add %s as this would exceed the %d device limit. Consider using the filter option",
-                state.entity_id,
-                MAX_DEVICES,
-            )
+        if self._would_exceed_max_devices(state.entity_id):
             return
 
         if state_needs_accessory_mode(state):
@@ -550,15 +600,13 @@ class HomeKit:
                 "The bridge %s has entity %s. For best performance, "
                 "and to prevent unexpected unavailability, create and "
                 "pair a separate HomeKit instance in accessory mode for "
-                "this entity.",
+                "this entity",
                 self._name,
                 state.entity_id,
             )
 
-        aid = self.hass.data[DOMAIN][self._entry_id][
-            AID_STORAGE
-        ].get_or_allocate_aid_for_entity_id(state.entity_id)
-        conf = self._config.pop(state.entity_id, {})
+        aid = self.aid_storage.get_or_allocate_aid_for_entity_id(state.entity_id)
+        conf = self._config.get(state.entity_id, {}).copy()
         # If an accessory cannot be created or added due to an exception
         # of any kind (usually in pyhap) it should not prevent
         # the rest of the accessories from being created
@@ -566,27 +614,59 @@ class HomeKit:
             acc = get_accessory(self.hass, self.driver, state, aid, conf)
             if acc is not None:
                 self.bridge.add_accessory(acc)
+                return acc
         except Exception:  # pylint: disable=broad-except
             _LOGGER.exception(
                 "Failed to create a HomeKit accessory for %s", state.entity_id
             )
+        return None
 
-    def remove_bridge_accessory(self, aid):
+    def _would_exceed_max_devices(self, name):
+        """Check if adding another devices would reach the limit and log."""
+        # The bridge itself counts as an accessory
+        if len(self.bridge.accessories) + 1 >= MAX_DEVICES:
+            _LOGGER.warning(
+                "Cannot add %s as this would exceed the %d device limit. Consider using the filter option",
+                name,
+                MAX_DEVICES,
+            )
+            return True
+        return False
+
+    def add_bridge_triggers_accessory(self, device, device_triggers):
+        """Add device automation triggers to the bridge."""
+        if self._would_exceed_max_devices(device.name):
+            return
+
+        aid = self.aid_storage.get_or_allocate_aid(device.id, device.id)
+        # If an accessory cannot be created or added due to an exception
+        # of any kind (usually in pyhap) it should not prevent
+        # the rest of the accessories from being created
+        config = {}
+        self._fill_config_from_device_registry_entry(device, config)
+        self.bridge.add_accessory(
+            DeviceTriggerAccessory(
+                self.hass,
+                self.driver,
+                device.name,
+                None,
+                aid,
+                config,
+                device_id=device.id,
+                device_triggers=device_triggers,
+            )
+        )
+
+    async def async_remove_bridge_accessory(self, aid):
         """Try adding accessory to bridge if configured beforehand."""
-        acc = None
-        if aid in self.bridge.accessories:
-            acc = self.bridge.accessories.pop(aid)
+        if acc := self.bridge.accessories.pop(aid, None):
+            await acc.stop()
         return acc
 
-    async def async_start(self, *args):
-        """Start the accessory driver."""
-        if self.status != STATUS_READY:
-            return
-        self.status = STATUS_WAIT
-
-        ent_reg = await entity_registry.async_get_registry(self.hass)
-        dev_reg = await device_registry.async_get_registry(self.hass)
-
+    async def async_configure_accessories(self):
+        """Configure accessories for the included states."""
+        dev_reg = device_registry.async_get(self.hass)
+        ent_reg = entity_registry.async_get(self.hass)
         device_lookup = ent_reg.async_get_device_class_lookup(
             {
                 (BINARY_SENSOR_DOMAIN, DEVICE_CLASS_BATTERY_CHARGING),
@@ -597,31 +677,70 @@ class HomeKit:
             }
         )
 
-        bridged_states = []
+        entity_states = []
         for state in self.hass.states.async_all():
             entity_id = state.entity_id
-
             if not self._filter(entity_id):
                 continue
 
-            ent_reg_ent = ent_reg.async_get(entity_id)
-            if ent_reg_ent:
+            if ent_reg_ent := ent_reg.async_get(entity_id):
                 await self._async_set_device_info_attributes(
                     ent_reg_ent, dev_reg, entity_id
                 )
                 self._async_configure_linked_sensors(ent_reg_ent, device_lookup, state)
 
-            bridged_states.append(state)
+            entity_states.append(state)
 
-        self._async_register_bridge(dev_reg)
-        await self._async_start(bridged_states)
+        return entity_states
+
+    async def async_start(self, *args):
+        """Load storage and start."""
+        if self.status != STATUS_READY:
+            return
+        self.status = STATUS_WAIT
+        async_zc_instance = await zeroconf.async_get_async_instance(self.hass)
+        uuid = await self.hass.helpers.instance_id.async_get()
+        await self.hass.async_add_executor_job(self.setup, async_zc_instance, uuid)
+        self.aid_storage = AccessoryAidStorage(self.hass, self._entry_id)
+        await self.aid_storage.async_initialize()
+        if not await self._async_create_accessories():
+            return
+        self._async_register_bridge()
         _LOGGER.debug("Driver start for %s", self._name)
-        self.hass.add_job(self.driver.start_service)
+        await self.driver.async_start()
+        async with self.hass.data[DOMAIN][PERSIST_LOCK]:
+            await self.hass.async_add_executor_job(self.driver.persist)
         self.status = STATUS_RUNNING
 
+        if self.driver.state.paired:
+            return
+        self._async_show_setup_message()
+
     @callback
-    def _async_register_bridge(self, dev_reg):
+    def _async_show_setup_message(self):
+        """Show the pairing setup message."""
+        async_show_setup_message(
+            self.hass,
+            self._entry_id,
+            accessory_friendly_name(self._entry_title, self.driver.accessory),
+            self.driver.state.pincode,
+            self.driver.accessory.xhm_uri(),
+        )
+
+    @callback
+    def async_unpair(self):
+        """Remove all pairings for an accessory so it can be repaired."""
+        state = self.driver.state
+        for client_uuid in list(state.paired_clients):
+            state.remove_paired_client(client_uuid)
+        self.driver.async_persist()
+        self.driver.async_update_advertisement()
+        self._async_show_setup_message()
+
+    @callback
+    def _async_register_bridge(self):
         """Register the bridge as a device so homekit_controller and exclude it from discovery."""
+        dev_reg = device_registry.async_get(self.hass)
         formatted_mac = device_registry.format_mac(self.driver.state.mac)
         # Connections and identifiers are both used here.
         #
@@ -645,8 +764,9 @@ class HomeKit:
             identifiers={identifier},
             connections={connection},
             manufacturer=MANUFACTURER,
-            name=self._name,
-            model=f"Home Assistant HomeKit {hk_mode_name}",
+            name=accessory_friendly_name(self._entry_title, self.driver.accessory),
+            model=f"HomeKit {hk_mode_name}",
+            entry_type=device_registry.DeviceEntryType.SERVICE,
         )
 
     @callback
@@ -663,30 +783,67 @@ class HomeKit:
         for device_id in devices_to_purge:
             dev_reg.async_remove_device(device_id)
 
-    async def _async_start(self, entity_states):
-        """Start the accessory."""
-        if self._homekit_mode == HOMEKIT_MODE_ACCESSORY:
-            state = entity_states[0]
-            conf = self._config.pop(state.entity_id, {})
-            acc = get_accessory(self.hass, self.driver, state, STANDALONE_AID, conf)
-
-            self.driver.add_accessory(acc)
-        else:
-            self.bridge = HomeBridge(self.hass, self.driver, self._name)
-            for state in entity_states:
-                self.add_bridge_accessory(state)
-            acc = self.bridge
-
-        await self.hass.async_add_executor_job(self.driver.add_accessory, acc)
-
-        if not self.driver.state.paired:
-            show_setup_message(
-                self.hass,
-                self._entry_id,
-                accessory_friendly_name(self._entry_title, self.driver.accessory),
-                self.driver.state.pincode,
-                self.driver.accessory.xhm_uri(),
+    @callback
+    def _async_create_single_accessory(self, entity_states):
+        """Create a single HomeKit accessory (accessory mode)."""
+        if not entity_states:
+            _LOGGER.error(
+                "HomeKit %s cannot startup: entity not available: %s",
+                self._name,
+                self._filter.config,
             )
+            return None
+        state = entity_states[0]
+        conf = self._config.get(state.entity_id, {}).copy()
+        acc = get_accessory(self.hass, self.driver, state, STANDALONE_AID, conf)
+        if acc is None:
+            _LOGGER.error(
+                "HomeKit %s cannot startup: entity not supported: %s",
+                self._name,
+                self._filter.config,
+            )
+        return acc
+
+    async def _async_create_bridge_accessory(self, entity_states):
+        """Create a HomeKit bridge with accessories. (bridge mode)."""
+        self.bridge = HomeBridge(self.hass, self.driver, self._name)
+        for state in entity_states:
+            self.add_bridge_accessory(state)
+        dev_reg = device_registry.async_get(self.hass)
+        if self._devices:
+            valid_device_ids = []
+            for device_id in self._devices:
+                if not dev_reg.async_get(device_id):
+                    _LOGGER.warning(
+                        "HomeKit %s cannot add device %s because it is missing from the device registry",
+                        self._name,
+                        device_id,
+                    )
+                else:
+                    valid_device_ids.append(device_id)
+            for device_id, device_triggers in (
+                await device_automation.async_get_device_automations(
+                    self.hass, "trigger", valid_device_ids
+                )
+            ).items():
+                self.add_bridge_triggers_accessory(
+                    dev_reg.async_get(device_id), device_triggers
+                )
+        return self.bridge
+
+    async def _async_create_accessories(self):
+        """Create the accessories."""
+        entity_states = await self.async_configure_accessories()
+        if self._homekit_mode == HOMEKIT_MODE_ACCESSORY:
+            acc = self._async_create_single_accessory(entity_states)
+        else:
+            acc = await self._async_create_bridge_accessory(entity_states)
+
+        if acc is None:
+            return False
+        # No need to load/persist as we do it in setup
+        self.driver.accessory = acc
+        return True
 
     async def async_stop(self, *args):
         """Stop the accessory driver."""
@@ -695,11 +852,6 @@ class HomeKit:
         self.status = STATUS_STOPPED
         _LOGGER.debug("Driver stop for %s", self._name)
         await self.driver.async_stop()
-        if self.bridge:
-            for acc in self.bridge.accessories.values():
-                acc.async_stop()
-        else:
-            self.driver.accessory.async_stop()
 
     @callback
     def _async_configure_linked_sensors(self, ent_reg_ent, device_lookup, state):
@@ -707,7 +859,7 @@ class HomeKit:
             ent_reg_ent is None
             or ent_reg_ent.device_id is None
             or ent_reg_ent.device_id not in device_lookup
-            or ent_reg_ent.device_class
+            or (ent_reg_ent.device_class or ent_reg_ent.original_device_class)
             in (DEVICE_CLASS_BATTERY_CHARGING, DEVICE_CLASS_BATTERY)
         ):
             return
@@ -763,23 +915,29 @@ class HomeKit:
         """Set attributes that will be used for homekit device info."""
         ent_cfg = self._config.setdefault(entity_id, {})
         if ent_reg_ent.device_id:
-            dev_reg_ent = dev_reg.async_get(ent_reg_ent.device_id)
-            if dev_reg_ent is not None:
-                # Handle missing devices
-                if dev_reg_ent.manufacturer:
-                    ent_cfg[ATTR_MANUFACTURER] = dev_reg_ent.manufacturer
-                if dev_reg_ent.model:
-                    ent_cfg[ATTR_MODEL] = dev_reg_ent.model
-                if dev_reg_ent.sw_version:
-                    ent_cfg[ATTR_SOFTWARE_VERSION] = dev_reg_ent.sw_version
+            if dev_reg_ent := dev_reg.async_get(ent_reg_ent.device_id):
+                self._fill_config_from_device_registry_entry(dev_reg_ent, ent_cfg)
         if ATTR_MANUFACTURER not in ent_cfg:
             try:
                 integration = await async_get_integration(
                     self.hass, ent_reg_ent.platform
                 )
-                ent_cfg[ATTR_INTERGRATION] = integration.name
+                ent_cfg[ATTR_INTEGRATION] = integration.name
             except IntegrationNotFound:
-                ent_cfg[ATTR_INTERGRATION] = ent_reg_ent.platform
+                ent_cfg[ATTR_INTEGRATION] = ent_reg_ent.platform
+
+    def _fill_config_from_device_registry_entry(self, device_entry, config):
+        """Populate a config dict from the registry."""
+        if device_entry.manufacturer:
+            config[ATTR_MANUFACTURER] = device_entry.manufacturer
+        if device_entry.model:
+            config[ATTR_MODEL] = device_entry.model
+        if device_entry.sw_version:
+            config[ATTR_SW_VERSION] = device_entry.sw_version
+        if device_entry.config_entries:
+            first_entry = list(device_entry.config_entries)[0]
+            if entry := self.hass.config_entries.async_get_entry(first_entry):
+                config[ATTR_INTEGRATION] = entry.domain
 
 
 class HomeKitPairingQRView(HomeAssistantView):
@@ -791,6 +949,7 @@ class HomeKitPairingQRView(HomeAssistantView):
 
     async def get(self, request):
         """Retrieve the pairing QRCode image."""
+        # pylint: disable=no-self-use
         if not request.query_string:
             raise Unauthorized()
         entry_id, secret = request.query_string.split("-")
