@@ -13,12 +13,9 @@ from aiohomekit.model.characteristics import (
 from aiohomekit.model.services import ServicesTypes
 from aiohomekit.utils import clamp_enum_to_char
 
-from homeassistant.components.climate import (
-    DEFAULT_MAX_HUMIDITY,
-    DEFAULT_MIN_HUMIDITY,
-    ClimateEntity,
-)
+from homeassistant.components.climate import ClimateEntity
 from homeassistant.components.climate.const import (
+    ATTR_HVAC_MODE,
     ATTR_TARGET_TEMP_HIGH,
     ATTR_TARGET_TEMP_LOW,
     CURRENT_HVAC_COOL,
@@ -92,8 +89,7 @@ async def async_setup_entry(hass, config_entry, async_add_entities):
 
     @callback
     def async_add_service(service):
-        entity_class = ENTITY_TYPES.get(service.short_type)
-        if not entity_class:
+        if not (entity_class := ENTITY_TYPES.get(service.short_type)):
             return False
         info = {"aid": service.accessory.aid, "iid": service.iid}
         async_add_entities([entity_class(conn, info)], True)
@@ -132,8 +128,8 @@ class HomeKitHeaterCoolerEntity(HomeKitEntity, ClimateEntity):
         else:
             hvac_mode = TARGET_HEATER_COOLER_STATE_HOMEKIT_TO_HASS.get(state)
             _LOGGER.warning(
-                "HomeKit device %s: Setting temperature in %s mode is not supported yet."
-                " Consider raising a ticket if you have this device and want to help us implement this feature.",
+                "HomeKit device %s: Setting temperature in %s mode is not supported yet;"
+                " Consider raising a ticket if you have this device and want to help us implement this feature",
                 self.entity_id,
                 hvac_mode,
             )
@@ -147,8 +143,8 @@ class HomeKitHeaterCoolerEntity(HomeKitEntity, ClimateEntity):
             return
         if hvac_mode not in {HVAC_MODE_HEAT, HVAC_MODE_COOL}:
             _LOGGER.warning(
-                "HomeKit device %s: Setting temperature in %s mode is not supported yet."
-                " Consider raising a ticket if you have this device and want to help us implement this feature.",
+                "HomeKit device %s: Setting temperature in %s mode is not supported yet;"
+                " Consider raising a ticket if you have this device and want to help us implement this feature",
                 self.entity_id,
                 hvac_mode,
             )
@@ -342,14 +338,27 @@ class HomeKitClimateEntity(HomeKitEntity, ClimateEntity):
 
     async def async_set_temperature(self, **kwargs):
         """Set new target temperature."""
+        chars = {}
+
+        value = self.service.value(CharacteristicsTypes.HEATING_COOLING_TARGET)
+        mode = MODE_HOMEKIT_TO_HASS.get(value)
+
+        if kwargs.get(ATTR_HVAC_MODE, mode) != mode:
+            mode = kwargs[ATTR_HVAC_MODE]
+            chars[CharacteristicsTypes.HEATING_COOLING_TARGET] = MODE_HASS_TO_HOMEKIT[
+                mode
+            ]
+
         temp = kwargs.get(ATTR_TEMPERATURE)
         heat_temp = kwargs.get(ATTR_TARGET_TEMP_LOW)
         cool_temp = kwargs.get(ATTR_TARGET_TEMP_HIGH)
-        value = self.service.value(CharacteristicsTypes.HEATING_COOLING_TARGET)
-        if MODE_HOMEKIT_TO_HASS.get(value) in {HVAC_MODE_HEAT_COOL}:
+
+        if (mode == HVAC_MODE_HEAT_COOL) and (
+            SUPPORT_TARGET_TEMPERATURE_RANGE & self.supported_features
+        ):
             if temp is None:
                 temp = (cool_temp + heat_temp) / 2
-            await self.async_put_characteristics(
+            chars.update(
                 {
                     CharacteristicsTypes.TEMPERATURE_HEATING_THRESHOLD: heat_temp,
                     CharacteristicsTypes.TEMPERATURE_COOLING_THRESHOLD: cool_temp,
@@ -357,9 +366,9 @@ class HomeKitClimateEntity(HomeKitEntity, ClimateEntity):
                 }
             )
         else:
-            await self.async_put_characteristics(
-                {CharacteristicsTypes.TEMPERATURE_TARGET: temp}
-            )
+            chars[CharacteristicsTypes.TEMPERATURE_TARGET] = temp
+
+        await self.async_put_characteristics(chars)
 
     async def async_set_humidity(self, humidity):
         """Set new target humidity."""
@@ -386,37 +395,54 @@ class HomeKitClimateEntity(HomeKitEntity, ClimateEntity):
     def target_temperature(self):
         """Return the temperature we try to reach."""
         value = self.service.value(CharacteristicsTypes.HEATING_COOLING_TARGET)
-        if MODE_HOMEKIT_TO_HASS.get(value) not in {HVAC_MODE_HEAT, HVAC_MODE_COOL}:
-            return None
-        return self.service.value(CharacteristicsTypes.TEMPERATURE_TARGET)
+        if (MODE_HOMEKIT_TO_HASS.get(value) in {HVAC_MODE_HEAT, HVAC_MODE_COOL}) or (
+            (MODE_HOMEKIT_TO_HASS.get(value) in {HVAC_MODE_HEAT_COOL})
+            and not (SUPPORT_TARGET_TEMPERATURE_RANGE & self.supported_features)
+        ):
+            return self.service.value(CharacteristicsTypes.TEMPERATURE_TARGET)
+        return None
 
     @property
     def target_temperature_high(self):
         """Return the highbound target temperature we try to reach."""
         value = self.service.value(CharacteristicsTypes.HEATING_COOLING_TARGET)
-        if MODE_HOMEKIT_TO_HASS.get(value) not in {HVAC_MODE_HEAT_COOL}:
-            return None
-        return self.service.value(CharacteristicsTypes.TEMPERATURE_COOLING_THRESHOLD)
+        if (MODE_HOMEKIT_TO_HASS.get(value) in {HVAC_MODE_HEAT_COOL}) and (
+            SUPPORT_TARGET_TEMPERATURE_RANGE & self.supported_features
+        ):
+            return self.service.value(
+                CharacteristicsTypes.TEMPERATURE_COOLING_THRESHOLD
+            )
+        return None
 
     @property
     def target_temperature_low(self):
         """Return the lowbound target temperature we try to reach."""
         value = self.service.value(CharacteristicsTypes.HEATING_COOLING_TARGET)
-        if MODE_HOMEKIT_TO_HASS.get(value) not in {HVAC_MODE_HEAT_COOL}:
-            return None
-        return self.service.value(CharacteristicsTypes.TEMPERATURE_HEATING_THRESHOLD)
+        if (MODE_HOMEKIT_TO_HASS.get(value) in {HVAC_MODE_HEAT_COOL}) and (
+            SUPPORT_TARGET_TEMPERATURE_RANGE & self.supported_features
+        ):
+            return self.service.value(
+                CharacteristicsTypes.TEMPERATURE_HEATING_THRESHOLD
+            )
+        return None
 
     @property
     def min_temp(self):
         """Return the minimum target temp."""
         value = self.service.value(CharacteristicsTypes.HEATING_COOLING_TARGET)
-        if MODE_HOMEKIT_TO_HASS.get(value) in {HVAC_MODE_HEAT_COOL}:
+        if (MODE_HOMEKIT_TO_HASS.get(value) in {HVAC_MODE_HEAT_COOL}) and (
+            SUPPORT_TARGET_TEMPERATURE_RANGE & self.supported_features
+        ):
             min_temp = self.service[
                 CharacteristicsTypes.TEMPERATURE_HEATING_THRESHOLD
             ].minValue
             if min_temp is not None:
                 return min_temp
-        if MODE_HOMEKIT_TO_HASS.get(value) in {HVAC_MODE_HEAT, HVAC_MODE_COOL}:
+        elif MODE_HOMEKIT_TO_HASS.get(value) in {
+            HVAC_MODE_HEAT,
+            HVAC_MODE_COOL,
+            HVAC_MODE_HEAT_COOL,
+        }:
             min_temp = self.service[CharacteristicsTypes.TEMPERATURE_TARGET].minValue
             if min_temp is not None:
                 return min_temp
@@ -426,13 +452,19 @@ class HomeKitClimateEntity(HomeKitEntity, ClimateEntity):
     def max_temp(self):
         """Return the maximum target temp."""
         value = self.service.value(CharacteristicsTypes.HEATING_COOLING_TARGET)
-        if MODE_HOMEKIT_TO_HASS.get(value) in {HVAC_MODE_HEAT_COOL}:
+        if (MODE_HOMEKIT_TO_HASS.get(value) in {HVAC_MODE_HEAT_COOL}) and (
+            SUPPORT_TARGET_TEMPERATURE_RANGE & self.supported_features
+        ):
             max_temp = self.service[
                 CharacteristicsTypes.TEMPERATURE_COOLING_THRESHOLD
             ].maxValue
             if max_temp is not None:
                 return max_temp
-        if MODE_HOMEKIT_TO_HASS.get(value) in {HVAC_MODE_HEAT, HVAC_MODE_COOL}:
+        elif MODE_HOMEKIT_TO_HASS.get(value) in {
+            HVAC_MODE_HEAT,
+            HVAC_MODE_COOL,
+            HVAC_MODE_HEAT_COOL,
+        }:
             max_temp = self.service[CharacteristicsTypes.TEMPERATURE_TARGET].maxValue
             if max_temp is not None:
                 return max_temp
@@ -451,14 +483,22 @@ class HomeKitClimateEntity(HomeKitEntity, ClimateEntity):
     @property
     def min_humidity(self):
         """Return the minimum humidity."""
-        char = self.service[CharacteristicsTypes.RELATIVE_HUMIDITY_TARGET]
-        return char.minValue or DEFAULT_MIN_HUMIDITY
+        min_humidity = self.service[
+            CharacteristicsTypes.RELATIVE_HUMIDITY_TARGET
+        ].minValue
+        if min_humidity is not None:
+            return min_humidity
+        return super().min_humidity
 
     @property
     def max_humidity(self):
         """Return the maximum humidity."""
-        char = self.service[CharacteristicsTypes.RELATIVE_HUMIDITY_TARGET]
-        return char.maxValue or DEFAULT_MAX_HUMIDITY
+        max_humidity = self.service[
+            CharacteristicsTypes.RELATIVE_HUMIDITY_TARGET
+        ].maxValue
+        if max_humidity is not None:
+            return max_humidity
+        return super().max_humidity
 
     @property
     def hvac_action(self):
