@@ -1,6 +1,7 @@
 """Support for Hue groups (room/zone)."""
 from __future__ import annotations
 
+import asyncio
 from typing import Any
 
 from aiohue.v2 import HueBridgeV2
@@ -18,6 +19,7 @@ from homeassistant.components.light import (
     COLOR_MODE_COLOR_TEMP,
     COLOR_MODE_ONOFF,
     COLOR_MODE_XY,
+    FLASH_SHORT,
     SUPPORT_FLASH,
     SUPPORT_TRANSITION,
     LightEntity,
@@ -29,14 +31,17 @@ from homeassistant.helpers.entity_platform import AddEntitiesCallback
 from ..bridge import HueBridge
 from ..const import CONF_ALLOW_HUE_GROUPS, DOMAIN
 from .entity import HueBaseEntity
-from .helpers import normalize_hue_brightness, normalize_hue_transition
+from .helpers import (
+    normalize_hue_brightness,
+    normalize_hue_colortemp,
+    normalize_hue_transition,
+)
 
 ALLOWED_ERRORS = [
     "device (groupedLight) has communication issues, command (on) may not have effect",
     'device (groupedLight) is "soft off", command (on) may not have effect',
     "device (light) has communication issues, command (on) may not have effect",
     'device (light) is "soft off", command (on) may not have effect',
-    "attribute (supportedAlertActions) cannot be written",
 ]
 
 
@@ -150,10 +155,15 @@ class GroupedHueLight(HueBaseEntity, LightEntity):
         """Turn the light on."""
         transition = normalize_hue_transition(kwargs.get(ATTR_TRANSITION))
         xy_color = kwargs.get(ATTR_XY_COLOR)
-        color_temp = kwargs.get(ATTR_COLOR_TEMP)
+        color_temp = normalize_hue_colortemp(kwargs.get(ATTR_COLOR_TEMP))
         brightness = normalize_hue_brightness(kwargs.get(ATTR_BRIGHTNESS))
         flash = kwargs.get(ATTR_FLASH)
 
+        if flash is not None:
+            await self.async_set_flash(flash)
+            # flash can not be sent with other commands at the same time
+            return
+
         # NOTE: a grouped_light can only handle turn on/off
         # To set other features, you'll have to control the attached lights
         if (
@@ -173,22 +183,32 @@ class GroupedHueLight(HueBaseEntity, LightEntity):
 
         # redirect all other feature commands to underlying lights
         # note that this silently ignores params sent to light that are not supported
-        for light in self.controller.get_lights(self.resource.id):
-            await self.bridge.async_request_call(
-                self.api.lights.set_state,
-                light.id,
-                on=True,
-                brightness=brightness if light.supports_dimming else None,
-                color_xy=xy_color if light.supports_color else None,
-                color_temp=color_temp if light.supports_color_temperature else None,
-                transition_time=transition,
-                alert=AlertEffectType.BREATHE if flash is not None else None,
-                allowed_errors=ALLOWED_ERRORS,
-            )
+        await asyncio.gather(
+            *[
+                self.bridge.async_request_call(
+                    self.api.lights.set_state,
+                    light.id,
+                    on=True,
+                    brightness=brightness if light.supports_dimming else None,
+                    color_xy=xy_color if light.supports_color else None,
+                    color_temp=color_temp if light.supports_color_temperature else None,
+                    transition_time=transition,
+                    alert=AlertEffectType.BREATHE if flash is not None else None,
+                    allowed_errors=ALLOWED_ERRORS,
+                )
+                for light in self.controller.get_lights(self.resource.id)
+            ]
+        )
 
     async def async_turn_off(self, **kwargs: Any) -> None:
         """Turn the light off."""
         transition = normalize_hue_transition(kwargs.get(ATTR_TRANSITION))
+        flash = kwargs.get(ATTR_FLASH)
+
+        if flash is not None:
+            await self.async_set_flash(flash)
+            # flash can not be sent with other commands at the same time
+            return
 
         # NOTE: a grouped_light can only handle turn on/off
         # To set other features, you'll have to control the attached lights
@@ -202,14 +222,31 @@ class GroupedHueLight(HueBaseEntity, LightEntity):
             return
 
         # redirect all other feature commands to underlying lights
-        for light in self.controller.get_lights(self.resource.id):
-            await self.bridge.async_request_call(
-                self.api.lights.set_state,
-                light.id,
-                on=False,
-                transition_time=transition,
-                allowed_errors=ALLOWED_ERRORS,
-            )
+        await asyncio.gather(
+            *[
+                self.bridge.async_request_call(
+                    self.api.lights.set_state,
+                    light.id,
+                    on=False,
+                    transition_time=transition,
+                    allowed_errors=ALLOWED_ERRORS,
+                )
+                for light in self.controller.get_lights(self.resource.id)
+            ]
+        )
+
+    async def async_set_flash(self, flash: str) -> None:
+        """Send flash command to light."""
+        await asyncio.gather(
+            *[
+                self.bridge.async_request_call(
+                    self.api.lights.set_flash,
+                    id=light.id,
+                    short=flash == FLASH_SHORT,
+                )
+                for light in self.controller.get_lights(self.resource.id)
+            ]
+        )
 
     @callback
     def on_update(self) -> None:
