@@ -1,7 +1,5 @@
 """The ONVIF integration."""
-import asyncio
-
-import voluptuous as vol
+from onvif.exceptions import ONVIFAuthError, ONVIFError, ONVIFTimeoutError
 
 from homeassistant.components.ffmpeg import CONF_EXTRA_ARGUMENTS
 from homeassistant.config_entries import SOURCE_IMPORT, ConfigEntry
@@ -12,13 +10,17 @@ from homeassistant.const import (
     CONF_PORT,
     CONF_USERNAME,
     EVENT_HOMEASSISTANT_STOP,
+    HTTP_BASIC_AUTHENTICATION,
+    HTTP_DIGEST_AUTHENTICATION,
 )
 from homeassistant.core import HomeAssistant
 from homeassistant.exceptions import ConfigEntryNotReady
 from homeassistant.helpers import config_per_platform
+from homeassistant.helpers.typing import ConfigType
 
 from .const import (
     CONF_RTSP_TRANSPORT,
+    CONF_SNAPSHOT_AUTH,
     DEFAULT_ARGUMENTS,
     DEFAULT_NAME,
     DEFAULT_PASSWORD,
@@ -29,10 +31,8 @@ from .const import (
 )
 from .device import ONVIFDevice
 
-CONFIG_SCHEMA = vol.Schema({DOMAIN: vol.Schema({})}, extra=vol.ALLOW_EXTRA)
 
-
-async def async_setup(hass: HomeAssistant, config: dict):
+async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool:
     """Set up the ONVIF component."""
     # Import from yaml
     configs = {}
@@ -41,7 +41,7 @@ async def async_setup(hass: HomeAssistant, config: dict):
             continue
 
         config = p_config.copy()
-        if config[CONF_HOST] not in configs.keys():
+        if config[CONF_HOST] not in configs:
             configs[config[CONF_HOST]] = {
                 CONF_HOST: config[CONF_HOST],
                 CONF_NAME: config.get(CONF_NAME, DEFAULT_NAME),
@@ -60,7 +60,7 @@ async def async_setup(hass: HomeAssistant, config: dict):
     return True
 
 
-async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry):
+async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Set up ONVIF from a config entry."""
     if DOMAIN not in hass.data:
         hass.data[DOMAIN] = {}
@@ -71,28 +71,32 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry):
     device = ONVIFDevice(hass, entry)
 
     if not await device.async_setup():
+        await device.device.close()
         return False
 
     if not device.available:
         raise ConfigEntryNotReady()
 
+    if not entry.data.get(CONF_SNAPSHOT_AUTH):
+        await async_populate_snapshot_auth(hass, device, entry)
+
     hass.data[DOMAIN][entry.unique_id] = device
 
     platforms = ["camera"]
 
-    if device.capabilities.events and await device.events.async_start():
+    if device.capabilities.events:
         platforms += ["binary_sensor", "sensor"]
-        hass.bus.async_listen_once(EVENT_HOMEASSISTANT_STOP, device.events.async_stop)
 
-    for component in platforms:
-        hass.async_create_task(
-            hass.config_entries.async_forward_entry_setup(entry, component)
-        )
+    hass.config_entries.async_setup_platforms(entry, platforms)
+
+    entry.async_on_unload(
+        hass.bus.async_listen_once(EVENT_HOMEASSISTANT_STOP, device.async_stop)
+    )
 
     return True
 
 
-async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry):
+async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Unload a config entry."""
 
     device = hass.data[DOMAIN][entry.unique_id]
@@ -102,14 +106,31 @@ async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry):
         platforms += ["binary_sensor", "sensor"]
         await device.events.async_stop()
 
-    return all(
-        await asyncio.gather(
-            *[
-                hass.config_entries.async_forward_entry_unload(entry, component)
-                for component in platforms
-            ]
-        )
-    )
+    return await hass.config_entries.async_unload_platforms(entry, platforms)
+
+
+async def _get_snapshot_auth(device):
+    """Determine auth type for snapshots."""
+    if not device.capabilities.snapshot or not (device.username and device.password):
+        return HTTP_DIGEST_AUTHENTICATION
+
+    try:
+        snapshot = await device.device.get_snapshot(device.profiles[0].token)
+
+        if snapshot:
+            return HTTP_DIGEST_AUTHENTICATION
+        return HTTP_BASIC_AUTHENTICATION
+    except (ONVIFAuthError, ONVIFTimeoutError):
+        return HTTP_BASIC_AUTHENTICATION
+    except ONVIFError:
+        return HTTP_DIGEST_AUTHENTICATION
+
+
+async def async_populate_snapshot_auth(hass, device, entry):
+    """Check if digest auth for snapshots is possible."""
+    auth = await _get_snapshot_auth(device)
+    new_data = {**entry.data, CONF_SNAPSHOT_AUTH: auth}
+    hass.config_entries.async_update_entry(entry, data=new_data)
 
 
 async def async_populate_options(hass, entry):
