@@ -23,16 +23,11 @@ from homeassistant.const import (
     STATE_OPEN,
     STATE_OPENING,
 )
+from homeassistant.core import callback
 import homeassistant.helpers.config_validation as cv
 
-from .const import (
-    ATTR_DISTANCE_SENSOR,
-    ATTR_DOOR_STATE,
-    ATTR_SIGNAL_STRENGTH,
-    CONF_DEVICE_KEY,
-    DEFAULT_PORT,
-    DOMAIN,
-)
+from .const import CONF_DEVICE_KEY, DEFAULT_PORT, DOMAIN
+from .entity import OpenGarageEntity
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -74,28 +69,22 @@ async def async_setup_platform(hass, config, async_add_entities, discovery_info=
 async def async_setup_entry(hass, entry, async_add_entities):
     """Set up the OpenGarage covers."""
     async_add_entities(
-        [OpenGarageCover(hass.data[DOMAIN][entry.entry_id], entry.unique_id)], True
+        [OpenGarageCover(hass.data[DOMAIN][entry.entry_id], entry.unique_id)]
     )
 
 
-class OpenGarageCover(CoverEntity):
+class OpenGarageCover(OpenGarageEntity, CoverEntity):
     """Representation of a OpenGarage cover."""
 
     _attr_device_class = DEVICE_CLASS_GARAGE
     _attr_supported_features = SUPPORT_OPEN | SUPPORT_CLOSE
 
-    def __init__(self, open_garage, device_id):
+    def __init__(self, open_garage_data_coordinator, device_id):
         """Initialize the cover."""
-        self._open_garage = open_garage
         self._state = None
         self._state_before_move = None
-        self._extra_state_attributes = {}
-        self._attr_unique_id = self._device_id = device_id
 
-    @property
-    def extra_state_attributes(self):
-        """Return the device state attributes."""
-        return self._extra_state_attributes
+        super().__init__(open_garage_data_coordinator, device_id)
 
     @property
     def is_closed(self):
@@ -134,16 +123,12 @@ class OpenGarageCover(CoverEntity):
         self._state = STATE_OPENING
         await self._push_button()
 
-    async def async_update(self):
-        """Get updated status from API."""
-        status = await self._open_garage.update_state()
-        if status is None:
-            _LOGGER.error("Unable to connect to OpenGarage device")
-            self._attr_available = False
-            return
+    @callback
+    def _update_attr(self) -> None:
+        """Update the state and attributes."""
+        status = self.coordinator.data
 
-        if self.name is None and status["name"] is not None:
-            self._attr_name = status["name"]
+        self._attr_name = status["name"]
         state = STATES_MAP.get(status.get("door"))
         if self._state_before_move is not None:
             if self._state_before_move != state:
@@ -152,19 +137,9 @@ class OpenGarageCover(CoverEntity):
         else:
             self._state = state
 
-        _LOGGER.debug("%s status: %s", self.name, self._state)
-        if status.get("rssi") is not None:
-            self._extra_state_attributes[ATTR_SIGNAL_STRENGTH] = status.get("rssi")
-        if status.get("dist") is not None:
-            self._extra_state_attributes[ATTR_DISTANCE_SENSOR] = status.get("dist")
-        if self._state is not None:
-            self._extra_state_attributes[ATTR_DOOR_STATE] = self._state
-
-        self._attr_available = True
-
     async def _push_button(self):
         """Send commands to API."""
-        result = await self._open_garage.push_button()
+        result = await self.coordinator.open_garage_connection.push_button()
         if result is None:
             _LOGGER.error("Unable to connect to OpenGarage device")
         if result == 1:
@@ -177,13 +152,3 @@ class OpenGarageCover(CoverEntity):
 
         self._state = self._state_before_move
         self._state_before_move = None
-
-    @property
-    def device_info(self):
-        """Return the device_info of the device."""
-        device_info = {
-            "identifiers": {(DOMAIN, self._device_id)},
-            "name": self.name,
-            "manufacturer": "Open Garage",
-        }
-        return device_info
