@@ -8,7 +8,7 @@ from homeassistant.const import DEVICE_CLASS_TIMESTAMP
 from homeassistant.core import callback
 from homeassistant.helpers.dispatcher import async_dispatcher_connect
 
-from . import REPETIER_API, SENSOR_TYPES, UPDATE_SIGNAL
+from . import REPETIER_API, SENSOR_TYPES, UPDATE_SIGNAL, RepetierSensorEntityDescription
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -35,12 +35,13 @@ def setup_platform(hass, config, add_entities, discovery_info=None):
         printer_id = info["printer_id"]
         sensor_type = info["sensor_type"]
         temp_id = info["temp_id"]
-        name = f"{info['name']}{SENSOR_TYPES[sensor_type][3]}"
+        description = SENSOR_TYPES[sensor_type]
+        name = f"{info['name']}{description.name or ''}"
         if temp_id is not None:
             _LOGGER.debug("%s Temp_id: %s", sensor_type, temp_id)
             name = f"{name}{temp_id}"
         sensor_class = sensor_map[sensor_type]
-        entity = sensor_class(api, temp_id, name, printer_id, sensor_type)
+        entity = sensor_class(api, temp_id, name, printer_id, description)
         entities.append(entity)
 
     add_entities(entities, True)
@@ -49,22 +50,27 @@ def setup_platform(hass, config, add_entities, discovery_info=None):
 class RepetierSensor(SensorEntity):
     """Class to create and populate a Repetier Sensor."""
 
-    def __init__(self, api, temp_id, name, printer_id, sensor_type):
-        """Init new sensor."""
-        self._api = api
-        self._attributes = {}
-        self._available = False
-        self._temp_id = temp_id
-        self._name = name
-        self._printer_id = printer_id
-        self._sensor_type = sensor_type
-        self._state = None
-        self._attr_device_class = SENSOR_TYPES[self._sensor_type][4]
+    entity_description: RepetierSensorEntityDescription
+    _attr_should_poll = False
 
-    @property
-    def available(self) -> bool:
-        """Return True if entity is available."""
-        return self._available
+    def __init__(
+        self,
+        api,
+        temp_id,
+        name,
+        printer_id,
+        description: RepetierSensorEntityDescription,
+    ):
+        """Init new sensor."""
+        self.entity_description = description
+        self._api = api
+        self._attributes: dict = {}
+        self._temp_id = temp_id
+        self._printer_id = printer_id
+        self._state = None
+
+        self._attr_name = name
+        self._attr_available = False
 
     @property
     def extra_state_attributes(self):
@@ -72,27 +78,7 @@ class RepetierSensor(SensorEntity):
         return self._attributes
 
     @property
-    def name(self):
-        """Return the name of the sensor."""
-        return self._name
-
-    @property
-    def unit_of_measurement(self):
-        """Return the unit of measurement of this entity, if any."""
-        return SENSOR_TYPES[self._sensor_type][1]
-
-    @property
-    def icon(self):
-        """Icon to use in the frontend."""
-        return SENSOR_TYPES[self._sensor_type][2]
-
-    @property
-    def should_poll(self):
-        """Return False as entity is updated from the component."""
-        return False
-
-    @property
-    def state(self):
+    def native_value(self):
         """Return sensor state."""
         return self._state
 
@@ -109,23 +95,21 @@ class RepetierSensor(SensorEntity):
 
     def _get_data(self):
         """Return new data from the api cache."""
-        data = self._api.get_data(self._printer_id, self._sensor_type, self._temp_id)
+        sensor_type = self.entity_description.key
+        data = self._api.get_data(self._printer_id, sensor_type, self._temp_id)
         if data is None:
-            _LOGGER.debug(
-                "Data not found for %s and %s", self._sensor_type, self._temp_id
-            )
-            self._available = False
+            _LOGGER.debug("Data not found for %s and %s", sensor_type, self._temp_id)
+            self._attr_available = False
             return None
-        self._available = True
+        self._attr_available = True
         return data
 
     def update(self):
         """Update the sensor."""
-        data = self._get_data()
-        if data is None:
+        if (data := self._get_data()) is None:
             return
         state = data.pop("state")
-        _LOGGER.debug("Printer %s State %s", self._name, state)
+        _LOGGER.debug("Printer %s State %s", self.name, state)
         self._attributes.update(data)
         self._state = state
 
@@ -134,7 +118,7 @@ class RepetierTempSensor(RepetierSensor):
     """Represent a Repetier temp sensor."""
 
     @property
-    def state(self):
+    def native_value(self):
         """Return sensor state."""
         if self._state is None:
             return None
@@ -142,12 +126,11 @@ class RepetierTempSensor(RepetierSensor):
 
     def update(self):
         """Update the sensor."""
-        data = self._get_data()
-        if data is None:
+        if (data := self._get_data()) is None:
             return
         state = data.pop("state")
         temp_set = data["temp_set"]
-        _LOGGER.debug("Printer %s Setpoint: %s, Temp: %s", self._name, temp_set, state)
+        _LOGGER.debug("Printer %s Setpoint: %s, Temp: %s", self.name, temp_set, state)
         self._attributes.update(data)
         self._state = state
 
@@ -156,7 +139,7 @@ class RepetierJobSensor(RepetierSensor):
     """Represent a Repetier job sensor."""
 
     @property
-    def state(self):
+    def native_value(self):
         """Return sensor state."""
         if self._state is None:
             return None
@@ -166,22 +149,18 @@ class RepetierJobSensor(RepetierSensor):
 class RepetierJobEndSensor(RepetierSensor):
     """Class to create and populate a Repetier Job End timestamp Sensor."""
 
-    @property
-    def device_class(self):
-        """Return the device class."""
-        return DEVICE_CLASS_TIMESTAMP
+    _attr_device_class = DEVICE_CLASS_TIMESTAMP
 
     def update(self):
         """Update the sensor."""
-        data = self._get_data()
-        if data is None:
+        if (data := self._get_data()) is None:
             return
         job_name = data["job_name"]
         start = data["start"]
         print_time = data["print_time"]
         from_start = data["from_start"]
         time_end = start + round(print_time, 0)
-        self._state = datetime.utcfromtimestamp(time_end).isoformat()
+        self._state = datetime.utcfromtimestamp(time_end)
         remaining = print_time - from_start
         remaining_secs = int(round(remaining, 0))
         _LOGGER.debug(
@@ -194,20 +173,16 @@ class RepetierJobEndSensor(RepetierSensor):
 class RepetierJobStartSensor(RepetierSensor):
     """Class to create and populate a Repetier Job Start timestamp Sensor."""
 
-    @property
-    def device_class(self):
-        """Return the device class."""
-        return DEVICE_CLASS_TIMESTAMP
+    _attr_device_class = DEVICE_CLASS_TIMESTAMP
 
     def update(self):
         """Update the sensor."""
-        data = self._get_data()
-        if data is None:
+        if (data := self._get_data()) is None:
             return
         job_name = data["job_name"]
         start = data["start"]
         from_start = data["from_start"]
-        self._state = datetime.utcfromtimestamp(start).isoformat()
+        self._state = datetime.utcfromtimestamp(start)
         elapsed_secs = int(round(from_start, 0))
         _LOGGER.debug(
             "Job %s elapsed %s",
