@@ -1,44 +1,19 @@
 """Constants for the Tuya integration."""
 from __future__ import annotations
 
+from collections.abc import Callable
 from dataclasses import dataclass, field
 from enum import Enum
-from typing import Callable
+import logging
 
 from tuya_iot import TuyaCloudOpenAPIEndpoint
 
+from homeassistant.components.sensor import SensorDeviceClass
 from homeassistant.const import (
     CONCENTRATION_MICROGRAMS_PER_CUBIC_METER,
     CONCENTRATION_MILLIGRAMS_PER_CUBIC_METER,
     CONCENTRATION_PARTS_PER_BILLION,
     CONCENTRATION_PARTS_PER_MILLION,
-    DEVICE_CLASS_AQI,
-    DEVICE_CLASS_BATTERY,
-    DEVICE_CLASS_CO,
-    DEVICE_CLASS_CO2,
-    DEVICE_CLASS_CURRENT,
-    DEVICE_CLASS_DATE,
-    DEVICE_CLASS_ENERGY,
-    DEVICE_CLASS_GAS,
-    DEVICE_CLASS_HUMIDITY,
-    DEVICE_CLASS_ILLUMINANCE,
-    DEVICE_CLASS_MONETARY,
-    DEVICE_CLASS_NITROGEN_DIOXIDE,
-    DEVICE_CLASS_NITROGEN_MONOXIDE,
-    DEVICE_CLASS_NITROUS_OXIDE,
-    DEVICE_CLASS_OZONE,
-    DEVICE_CLASS_PM1,
-    DEVICE_CLASS_PM10,
-    DEVICE_CLASS_PM25,
-    DEVICE_CLASS_POWER,
-    DEVICE_CLASS_POWER_FACTOR,
-    DEVICE_CLASS_PRESSURE,
-    DEVICE_CLASS_SIGNAL_STRENGTH,
-    DEVICE_CLASS_SULPHUR_DIOXIDE,
-    DEVICE_CLASS_TEMPERATURE,
-    DEVICE_CLASS_TIMESTAMP,
-    DEVICE_CLASS_VOLATILE_ORGANIC_COMPOUNDS,
-    DEVICE_CLASS_VOLTAGE,
     ELECTRIC_CURRENT_AMPERE,
     ELECTRIC_CURRENT_MILLIAMPERE,
     ELECTRIC_POTENTIAL_MILLIVOLT,
@@ -64,6 +39,7 @@ from homeassistant.const import (
 )
 
 DOMAIN = "tuya"
+LOGGER = logging.getLogger(__package__)
 
 CONF_AUTH_TYPE = "auth_type"
 CONF_PROJECT_TYPE = "tuya_project_type"
@@ -85,6 +61,10 @@ DEVICE_CLASS_TUYA_MOTION_SENSITIVITY = "tuya__motion_sensitivity"
 DEVICE_CLASS_TUYA_RECORD_MODE = "tuya__record_mode"
 DEVICE_CLASS_TUYA_RELAY_STATUS = "tuya__relay_status"
 DEVICE_CLASS_TUYA_STATUS = "tuya__status"
+DEVICE_CLASS_TUYA_FINGERBOT_MODE = "tuya__fingerbot_mode"
+DEVICE_CLASS_TUYA_VACUUM_CISTERN = "tuya__vacuum_cistern"
+DEVICE_CLASS_TUYA_VACUUM_COLLECTION = "tuya__vacuum_collection"
+DEVICE_CLASS_TUYA_VACUUM_MODE = "tuya__vacuum_mode"
 
 TUYA_DISCOVERY_NEW = "tuya_discovery_new"
 TUYA_HA_SIGNAL_UPDATE_ENTITY = "tuya_entry_update"
@@ -100,6 +80,7 @@ SMARTLIFE_APP = "smartlife"
 
 PLATFORMS = [
     "binary_sensor",
+    "button",
     "camera",
     "climate",
     "cover",
@@ -137,6 +118,8 @@ class DPCode(str, Enum):
     ANGLE_HORIZONTAL = "angle_horizontal"
     ANGLE_VERTICAL = "angle_vertical"
     ANION = "anion"  # Ionizer unit
+    ARM_DOWN_PERCENT = "arm_down_percent"
+    ARM_UP_PERCENT = "arm_up_percent"
     BASIC_ANTI_FLICKER = "basic_anti_flicker"
     BASIC_DEVICE_VOLUME = "basic_device_volume"
     BASIC_FLIP = "basic_flip"
@@ -167,14 +150,19 @@ class DPCode(str, Enum):
     CH4_SENSOR_STATE = "ch4_sensor_state"
     CH4_SENSOR_VALUE = "ch4_sensor_value"
     CHILD_LOCK = "child_lock"  # Child lock
+    CISTERN = "cistern"
+    CLEAN_AREA = "clean_area"
+    CLEAN_TIME = "clean_time"
+    CLICK_SUSTAIN_TIME = "click_sustain_time"
     CO_STATE = "co_state"
     CO_STATUS = "co_status"
     CO_VALUE = "co_value"
     CO2_STATE = "co2_state"
     CO2_VALUE = "co2_value"  # CO2 concentration
+    COLLECTION_MODE = "collection_mode"
     COLOR_DATA_V2 = "color_data_v2"
-    COLOUR_DATA_HSV = "colour_data_hsv"  # Colored light mode
     COLOUR_DATA = "colour_data"  # Colored light mode
+    COLOUR_DATA_HSV = "colour_data_hsv"  # Colored light mode
     COLOUR_DATA_V2 = "colour_data_v2"  # Colored light mode
     CONCENTRATION_SET = "concentration_set"  # Concentration setting
     CONTROL = "control"
@@ -190,16 +178,21 @@ class DPCode(str, Enum):
     DEHUMIDITY_SET_VALUE = "dehumidify_set_value"
     DO_NOT_DISTURB = "do_not_disturb"
     DOORCONTACT_STATE = "doorcontact_state"  # Status of door window sensor
-    DOORCONTACT_STATE_2 = "doorcontact_state_3"
+    DOORCONTACT_STATE_2 = "doorcontact_state_2"
     DOORCONTACT_STATE_3 = "doorcontact_state_3"
+    DUSTER_CLOTH = "duster_cloth"
+    EDGE_BRUSH = "edge_brush"
     ELECTRICITY_LEFT = "electricity_left"
     FAN_DIRECTION = "fan_direction"  # Fan direction
     FAN_SPEED_ENUM = "fan_speed_enum"  # Speed mode
     FAN_SPEED_PERCENT = "fan_speed_percent"  # Stepless speed
     FAR_DETECTION = "far_detection"
+    FAULT = "fault"
+    FILTER_LIFE = "filter"
     FILTER_RESET = "filter_reset"  # Filter (cartridge) reset
     FLOODLIGHT_LIGHTNESS = "floodlight_lightness"
     FLOODLIGHT_SWITCH = "floodlight_switch"
+    FORWARD_ENERGY_TOTAL = "forward_energy_total"
     GAS_SENSOR_STATE = "gas_sensor_state"
     GAS_SENSOR_STATUS = "gas_sensor_status"
     GAS_SENSOR_VALUE = "gas_sensor_value"
@@ -229,12 +222,16 @@ class DPCode(str, Enum):
     PERCENT_STATE = "percent_state"
     PERCENT_STATE_2 = "percent_state_2"
     PERCENT_STATE_3 = "percent_state_3"
+    PHASE_A = "phase_a"
+    PHASE_B = "phase_b"
+    PHASE_C = "phase_c"
     PIR = "pir"  # Motion sensor
     PM1 = "pm1"
     PM10 = "pm10"
     PM25_STATE = "pm25_state"
     PM25_VALUE = "pm25_value"
     POWDER_SET = "powder_set"  # Powder
+    POWER = "power"
     POWER_GO = "power_go"
     PRESENCE_STATE = "presence_state"
     PRESSURE_STATE = "pressure_state"
@@ -243,6 +240,12 @@ class DPCode(str, Enum):
     RECORD_MODE = "record_mode"
     RECORD_SWITCH = "record_switch"  # Recording switch
     RELAY_STATUS = "relay_status"
+    RESET_DUSTER_CLOTH = "reset_duster_cloth"
+    RESET_EDGE_BRUSH = "reset_edge_brush"
+    RESET_FILTER = "reset_filter"
+    RESET_MAP = "reset_map"
+    RESET_ROLL_BRUSH = "reset_roll_brush"
+    ROLL_BRUSH = "roll_brush"
     SEEK = "seek"
     SENSITIVITY = "sensitivity"  # Sensitivity
     SENSOR_HUMIDITY = "sensor_humidity"
@@ -271,6 +274,7 @@ class DPCode(str, Enum):
     SWITCH_BACKLIGHT = "switch_backlight"  # Backlight switch
     SWITCH_CHARGE = "switch_charge"
     SWITCH_CONTROLLER = "switch_controller"
+    SWITCH_DISTURB = "switch_disturb"
     SWITCH_HORIZONTAL = "switch_horizontal"  # Horizontal swing flap switch
     SWITCH_LED = "switch_led"  # Switch
     SWITCH_LED_1 = "switch_led_1"
@@ -298,12 +302,17 @@ class DPCode(str, Enum):
     TEMP_VALUE = "temp_value"  # Color temperature
     TEMP_VALUE_V2 = "temp_value_v2"
     TEMPER_ALARM = "temper_alarm"  # Tamper alarm
+    TOTAL_CLEAN_AREA = "total_clean_area"
+    TOTAL_CLEAN_COUNT = "total_clean_count"
+    TOTAL_CLEAN_TIME = "total_clean_time"
     UV = "uv"  # UV sterilization
     VA_BATTERY = "va_battery"
     VA_HUMIDITY = "va_humidity"
     VA_TEMPERATURE = "va_temperature"
     VOC_STATE = "voc_state"
     VOC_VALUE = "voc_value"
+    VOICE_SWITCH = "voice_switch"
+    VOLUME_SET = "volume_set"
     WARM = "warm"  # Heat preservation
     WARM_TIME = "warm_time"  # Heat preservation time
     WATER_RESET = "water_reset"  # Resetting of water usage days
@@ -337,33 +346,33 @@ UNITS = (
         unit="",
         aliases={" "},
         device_classes={
-            DEVICE_CLASS_AQI,
-            DEVICE_CLASS_DATE,
-            DEVICE_CLASS_MONETARY,
-            DEVICE_CLASS_TIMESTAMP,
+            SensorDeviceClass.AQI,
+            SensorDeviceClass.DATE,
+            SensorDeviceClass.MONETARY,
+            SensorDeviceClass.TIMESTAMP,
         },
     ),
     UnitOfMeasurement(
         unit=PERCENTAGE,
         aliases={"pct", "percent"},
         device_classes={
-            DEVICE_CLASS_BATTERY,
-            DEVICE_CLASS_HUMIDITY,
-            DEVICE_CLASS_POWER_FACTOR,
+            SensorDeviceClass.BATTERY,
+            SensorDeviceClass.HUMIDITY,
+            SensorDeviceClass.POWER_FACTOR,
         },
     ),
     UnitOfMeasurement(
         unit=CONCENTRATION_PARTS_PER_MILLION,
         device_classes={
-            DEVICE_CLASS_CO,
-            DEVICE_CLASS_CO2,
+            SensorDeviceClass.CO,
+            SensorDeviceClass.CO2,
         },
     ),
     UnitOfMeasurement(
         unit=CONCENTRATION_PARTS_PER_BILLION,
         device_classes={
-            DEVICE_CLASS_CO,
-            DEVICE_CLASS_CO2,
+            SensorDeviceClass.CO,
+            SensorDeviceClass.CO2,
         },
         conversion_unit=CONCENTRATION_PARTS_PER_MILLION,
         conversion_fn=lambda x: x / 1000,
@@ -371,73 +380,73 @@ UNITS = (
     UnitOfMeasurement(
         unit=ELECTRIC_CURRENT_AMPERE,
         aliases={"a", "ampere"},
-        device_classes={DEVICE_CLASS_CURRENT},
+        device_classes={SensorDeviceClass.CURRENT},
     ),
     UnitOfMeasurement(
         unit=ELECTRIC_CURRENT_MILLIAMPERE,
         aliases={"ma", "milliampere"},
-        device_classes={DEVICE_CLASS_CURRENT},
+        device_classes={SensorDeviceClass.CURRENT},
         conversion_unit=ELECTRIC_CURRENT_AMPERE,
         conversion_fn=lambda x: x / 1000,
     ),
     UnitOfMeasurement(
         unit=ENERGY_WATT_HOUR,
         aliases={"wh", "watthour"},
-        device_classes={DEVICE_CLASS_ENERGY},
+        device_classes={SensorDeviceClass.ENERGY},
     ),
     UnitOfMeasurement(
         unit=ENERGY_KILO_WATT_HOUR,
-        aliases={"kwh", "kilowatt-hour"},
-        device_classes={DEVICE_CLASS_ENERGY},
+        aliases={"kwh", "kilowatt-hour", "kW·h"},
+        device_classes={SensorDeviceClass.ENERGY},
     ),
     UnitOfMeasurement(
         unit=VOLUME_CUBIC_FEET,
         aliases={"ft3"},
-        device_classes={DEVICE_CLASS_GAS},
+        device_classes={SensorDeviceClass.GAS},
     ),
     UnitOfMeasurement(
         unit=VOLUME_CUBIC_METERS,
         aliases={"m3"},
-        device_classes={DEVICE_CLASS_GAS},
+        device_classes={SensorDeviceClass.GAS},
     ),
     UnitOfMeasurement(
         unit=LIGHT_LUX,
         aliases={"lux"},
-        device_classes={DEVICE_CLASS_ILLUMINANCE},
+        device_classes={SensorDeviceClass.ILLUMINANCE},
     ),
     UnitOfMeasurement(
         unit="lm",
         aliases={"lum", "lumen"},
-        device_classes={DEVICE_CLASS_ILLUMINANCE},
+        device_classes={SensorDeviceClass.ILLUMINANCE},
     ),
     UnitOfMeasurement(
         unit=CONCENTRATION_MICROGRAMS_PER_CUBIC_METER,
         aliases={"ug/m3", "µg/m3", "ug/m³"},
         device_classes={
-            DEVICE_CLASS_NITROGEN_DIOXIDE,
-            DEVICE_CLASS_NITROGEN_MONOXIDE,
-            DEVICE_CLASS_NITROUS_OXIDE,
-            DEVICE_CLASS_OZONE,
-            DEVICE_CLASS_PM1,
-            DEVICE_CLASS_PM25,
-            DEVICE_CLASS_PM10,
-            DEVICE_CLASS_SULPHUR_DIOXIDE,
-            DEVICE_CLASS_VOLATILE_ORGANIC_COMPOUNDS,
+            SensorDeviceClass.NITROGEN_DIOXIDE,
+            SensorDeviceClass.NITROGEN_MONOXIDE,
+            SensorDeviceClass.NITROUS_OXIDE,
+            SensorDeviceClass.OZONE,
+            SensorDeviceClass.PM1,
+            SensorDeviceClass.PM25,
+            SensorDeviceClass.PM10,
+            SensorDeviceClass.SULPHUR_DIOXIDE,
+            SensorDeviceClass.VOLATILE_ORGANIC_COMPOUNDS,
         },
     ),
     UnitOfMeasurement(
         unit=CONCENTRATION_MILLIGRAMS_PER_CUBIC_METER,
         aliases={"mg/m3"},
         device_classes={
-            DEVICE_CLASS_NITROGEN_DIOXIDE,
-            DEVICE_CLASS_NITROGEN_MONOXIDE,
-            DEVICE_CLASS_NITROUS_OXIDE,
-            DEVICE_CLASS_OZONE,
-            DEVICE_CLASS_PM1,
-            DEVICE_CLASS_PM25,
-            DEVICE_CLASS_PM10,
-            DEVICE_CLASS_SULPHUR_DIOXIDE,
-            DEVICE_CLASS_VOLATILE_ORGANIC_COMPOUNDS,
+            SensorDeviceClass.NITROGEN_DIOXIDE,
+            SensorDeviceClass.NITROGEN_MONOXIDE,
+            SensorDeviceClass.NITROUS_OXIDE,
+            SensorDeviceClass.OZONE,
+            SensorDeviceClass.PM1,
+            SensorDeviceClass.PM25,
+            SensorDeviceClass.PM10,
+            SensorDeviceClass.SULPHUR_DIOXIDE,
+            SensorDeviceClass.VOLATILE_ORGANIC_COMPOUNDS,
         },
         conversion_unit=CONCENTRATION_MICROGRAMS_PER_CUBIC_METER,
         conversion_fn=lambda x: x * 1000,
@@ -445,69 +454,69 @@ UNITS = (
     UnitOfMeasurement(
         unit=POWER_WATT,
         aliases={"watt"},
-        device_classes={DEVICE_CLASS_POWER},
+        device_classes={SensorDeviceClass.POWER},
     ),
     UnitOfMeasurement(
         unit=POWER_KILO_WATT,
         aliases={"kilowatt"},
-        device_classes={DEVICE_CLASS_POWER},
+        device_classes={SensorDeviceClass.POWER},
     ),
     UnitOfMeasurement(
         unit=PRESSURE_BAR,
-        device_classes={DEVICE_CLASS_PRESSURE},
+        device_classes={SensorDeviceClass.PRESSURE},
     ),
     UnitOfMeasurement(
         unit=PRESSURE_MBAR,
         aliases={"millibar"},
-        device_classes={DEVICE_CLASS_PRESSURE},
+        device_classes={SensorDeviceClass.PRESSURE},
     ),
     UnitOfMeasurement(
         unit=PRESSURE_HPA,
         aliases={"hpa", "hectopascal"},
-        device_classes={DEVICE_CLASS_PRESSURE},
+        device_classes={SensorDeviceClass.PRESSURE},
     ),
     UnitOfMeasurement(
         unit=PRESSURE_INHG,
         aliases={"inhg"},
-        device_classes={DEVICE_CLASS_PRESSURE},
+        device_classes={SensorDeviceClass.PRESSURE},
     ),
     UnitOfMeasurement(
         unit=PRESSURE_PSI,
-        device_classes={DEVICE_CLASS_PRESSURE},
+        device_classes={SensorDeviceClass.PRESSURE},
     ),
     UnitOfMeasurement(
         unit=PRESSURE_PA,
-        device_classes={DEVICE_CLASS_PRESSURE},
+        device_classes={SensorDeviceClass.PRESSURE},
     ),
     UnitOfMeasurement(
         unit=SIGNAL_STRENGTH_DECIBELS,
         aliases={"db"},
-        device_classes={DEVICE_CLASS_SIGNAL_STRENGTH},
+        device_classes={SensorDeviceClass.SIGNAL_STRENGTH},
     ),
     UnitOfMeasurement(
         unit=SIGNAL_STRENGTH_DECIBELS_MILLIWATT,
         aliases={"dbm"},
-        device_classes={DEVICE_CLASS_SIGNAL_STRENGTH},
+        device_classes={SensorDeviceClass.SIGNAL_STRENGTH},
     ),
     UnitOfMeasurement(
         unit=TEMP_CELSIUS,
         aliases={"°c", "c", "celsius"},
-        device_classes={DEVICE_CLASS_TEMPERATURE},
+        device_classes={SensorDeviceClass.TEMPERATURE},
     ),
     UnitOfMeasurement(
         unit=TEMP_FAHRENHEIT,
         aliases={"°f", "f", "fahrenheit"},
-        device_classes={DEVICE_CLASS_TEMPERATURE},
+        device_classes={SensorDeviceClass.TEMPERATURE},
     ),
     UnitOfMeasurement(
         unit=ELECTRIC_POTENTIAL_VOLT,
         aliases={"volt"},
-        device_classes={DEVICE_CLASS_VOLTAGE},
+        device_classes={SensorDeviceClass.VOLTAGE},
     ),
     UnitOfMeasurement(
         unit=ELECTRIC_POTENTIAL_MILLIVOLT,
         aliases={"mv", "millivolt"},
-        device_classes={DEVICE_CLASS_VOLTAGE},
+        device_classes={SensorDeviceClass.VOLTAGE},
         conversion_unit=ELECTRIC_POTENTIAL_VOLT,
         conversion_fn=lambda x: x / 1000,
     ),
