@@ -2,15 +2,19 @@
 from __future__ import annotations
 
 import asyncio
+from collections.abc import Callable
 import functools
 import logging
 from traceback import FrameSummary, extract_stack
-from typing import Any, Callable, TypeVar, cast
+from typing import Any, TypeVar, cast
 
 from homeassistant.exceptions import HomeAssistantError
 
 _LOGGER = logging.getLogger(__name__)
 
+# Keep track of integrations already reported to prevent flooding
+_REPORTED_INTEGRATIONS: set[str] = set()
+
 CALLABLE_T = TypeVar("CALLABLE_T", bound=Callable)  # pylint: disable=invalid-name
 
 
@@ -49,24 +53,34 @@ class MissingIntegrationFrame(HomeAssistantError):
     """Raised when no integration is found in the frame."""
 
 
-def report(what: str) -> None:
+def report(
+    what: str,
+    exclude_integrations: set | None = None,
+    error_if_core: bool = True,
+    level: int = logging.WARNING,
+) -> None:
     """Report incorrect usage.
 
     Async friendly.
     """
     try:
-        integration_frame = get_integration_frame()
+        integration_frame = get_integration_frame(
+            exclude_integrations=exclude_integrations
+        )
     except MissingIntegrationFrame as err:
-        # Did not source from an integration? Hard error.
-        raise RuntimeError(
-            f"Detected code that {what}. Please report this issue."
-        ) from err
+        msg = f"Detected code that {what}. Please report this issue."
+        if error_if_core:
+            raise RuntimeError(msg) from err
+        _LOGGER.warning(msg, stack_info=True)
+        return
 
-    report_integration(what, integration_frame)
+    report_integration(what, integration_frame, level)
 
 
 def report_integration(
-    what: str, integration_frame: tuple[FrameSummary, str, str]
+    what: str,
+    integration_frame: tuple[FrameSummary, str, str],
+    level: int = logging.WARNING,
 ) -> None:
     """Report incorrect usage in an integration.
 
@@ -74,13 +88,20 @@ def report_integration(
     """
     found_frame, integration, path = integration_frame
 
+    # Keep track of integrations already reported to prevent flooding
+    key = f"{found_frame.filename}:{found_frame.lineno}"
+    if key in _REPORTED_INTEGRATIONS:
+        return
+    _REPORTED_INTEGRATIONS.add(key)
+
     index = found_frame.filename.index(path)
     if path == "custom_components/":
         extra = " to the custom component author"
     else:
         extra = ""
 
-    _LOGGER.warning(
+    _LOGGER.log(
+        level,
         "Detected integration that %s. "
         "Please report issue%s for %s using this method at %s, line %s: %s",
         what,
